/*
 * This example sketch shows how you can manage the nRF5 pin mapping as part of your code.
 * You can use the method for any nRF51822 or nRF52832 board or module.
 * 
 * Most components, like UART, SPI, Wire Bus, of the nRF5 series chips don't
 * have a fixed pin mapping. There are some pins with restrictions like analog
 * inputs, NFC or pins near the radio module. Please refer the latest
 * documentation about pin restrictions at http://infocenter.nordicsemi.com 
 * 
 * To use the custom pin mapping you have to do following steps:
 * 
 * 1. Install "arduino-nrf5" like described at
 *    https://github.com/sandeepmistry/arduino-nRF5/
 * 2. Install the "My Sensors nRF5 Boards" with the board manager like
 *    explained at https://github.com/mysensors/ArduinoBoards
 * 3. Copy the files "MyBoardNRF5.cpp" and "MyBoardNRF5.h" from
 *    "MyBoardNRF5" example into your sketch.
 * 4. Modify pin mappings in "MyBoardNRF5.cpp" and "MyBoardNRF5.h" to fit
 *    your requirements.
 * 5. Select "MyBoardNRF5 nrf52832" or "MyBoardNRF5 nrf52822" as your board.
 *    Choose the correct parameters and programmer in the Tools menu.
 */

/* Module DIN 3xRelays
 *  LED verte Power
 *  LED Rouge état relay (3x)
 *    P0.20 -> Relay1
 *    P0.19 -> Relay2
 *    P0.18 -> Relay3
 *  LED Bleu Radio Communication
 *    P0.22 -> Comunication
 */

// What so ever you need to setup
#define MY_NODE_ID 10

// Inverses the behavior of leds
#define MY_WITH_LEDS_BLINKING_INVERSE

// Flash leds on rx/tx/err
#define MY_DEFAULT_RX_LED_PIN  22  // Receive led pin
#define MY_DEFAULT_TX_LED_PIN  22  // Transmit led pin

#define INTER_MESSAGES_WAIT 50

#include <MySensors.h>

MyMessage relay1Msg(0, V_STATUS);
MyMessage relay2Msg(1, V_STATUS);
MyMessage relay3Msg(2, V_STATUS);


#define RELAY1 20 
#define RELAY2 19 
#define RELAY3 18

#define RELAY_ON  0
#define RELAY_OFF 1

#define RELAY1_CHILD_ID 0
#define RELAY2_CHILD_ID 1
#define RELAY3_CHILD_ID 2 

int Count = 0;


//-------------------------------------
void nRF_Init() {
  NRF_POWER->DCDCEN = 0; // Not reducing the radio current   
  NRF_PWM0  ->ENABLE = 0;
  NRF_PWM1  ->ENABLE = 0;
  NRF_PWM2  ->ENABLE = 0;
  NRF_TWIM1 ->ENABLE = 0;
  NRF_TWIS1 ->ENABLE = 0;
  //NRF_RADIO->TXPOWER = 8;
}

void disableNfc() {
  NRF_NFCT->TASKS_DISABLE = 1;
  NRF_NVMC->CONFIG = 1;
  NRF_UICR->NFCPINS = 0;
  NRF_NVMC->CONFIG = 0;
}

void turnOffAdc() {
  if (NRF_SAADC->ENABLE) {
    NRF_SAADC->TASKS_STOP = 1;
    while (NRF_SAADC->EVENTS_STOPPED) {}
    NRF_SAADC->ENABLE = 0;
    while (NRF_SAADC->ENABLE) {}
  }
}

void preHwInit() {
  //hwPinMode(LED_BUILTIN,OUTPUT_D0H1);
  pinMode(RELAY1, OUTPUT); 
  pinMode(RELAY2, OUTPUT);
  pinMode(RELAY3, OUTPUT); 
  pinMode(22, OUTPUT);   
}

void before()
{  
  nRF_Init();
  disableNfc();
  turnOffAdc();       
  wait(200);
}

void setup() {  
  // Initialise RELAY(s)
  pinMode(RELAY1, OUTPUT);
  digitalWrite(RELAY1, RELAY_OFF);
  digitalWrite(RELAY1, loadState(RELAY1_CHILD_ID)?RELAY_ON:RELAY_OFF);

  pinMode(RELAY2, OUTPUT);
  digitalWrite(RELAY2, RELAY_OFF);
  digitalWrite(RELAY2, loadState(RELAY2_CHILD_ID)?RELAY_ON:RELAY_OFF);

  pinMode(RELAY3, OUTPUT);
  digitalWrite(RELAY3, RELAY_OFF);
  digitalWrite(RELAY3, loadState(RELAY3_CHILD_ID)?RELAY_ON:RELAY_OFF);

  NRF_WDT->CONFIG = NRF_WDT->CONFIG = (WDT_CONFIG_HALT_Pause << WDT_CONFIG_HALT_Pos) | ( WDT_CONFIG_SLEEP_Run << WDT_CONFIG_SLEEP_Pos); 
  NRF_WDT->CRV = (32768*70000)/1000; 
  NRF_WDT->RREN |= WDT_RREN_RR0_Msk;  
  NRF_WDT->TASKS_START = 1;  
}

void presentation()
{
  sendSketchInfo("DIN 3xRELAYS", "0.8");
  wait(INTER_MESSAGES_WAIT);  
  present(RELAY1_CHILD_ID, S_BINARY, "Relay 1");  
  wait(INTER_MESSAGES_WAIT); 
  present(RELAY2_CHILD_ID, S_BINARY, "Relay 2");  
  wait(INTER_MESSAGES_WAIT); 
  present(RELAY3_CHILD_ID, S_BINARY, "Relay 3");
  wait(INTER_MESSAGES_WAIT);   
}

void loop() {
  NRF_WDT->RR[0] = WDT_RR_RR_Reload;

  Count = 0;
  sendBatteryLevel(100);
  wait(60000);
}

void receive(const MyMessage &message)
{
  if ((message.type == V_STATUS) && (message.sensor == RELAY1_CHILD_ID))
  {
    digitalWrite(RELAY1, message.getBool()?RELAY_ON:RELAY_OFF);
    send(relay1Msg.set(message.getBool())); 
    saveState(RELAY1_CHILD_ID,(uint8_t)message.getBool());
    #ifdef MY_DEBUG
      Serial.print("--Incoming change for relay 1: ");
      Serial.println(message.getBool());
    #endif
  }

  if ((message.type == V_STATUS) && (message.sensor == RELAY2_CHILD_ID))
  {
    digitalWrite(RELAY2, message.getBool()?RELAY_ON:RELAY_OFF);
    send(relay2Msg.set(message.getBool())); 
    saveState(RELAY2_CHILD_ID,(uint8_t)message.getBool());
    #ifdef MY_DEBUG
      Serial.print("--Incoming change for relay 2: ");
      Serial.println(message.getBool());
    #endif
  }

  if ((message.type == V_STATUS) && (message.sensor == RELAY3_CHILD_ID))
  {
    digitalWrite(RELAY3, message.getBool()?RELAY_ON:RELAY_OFF);
    send(relay3Msg.set(message.getBool())); 
    saveState(RELAY3_CHILD_ID,(uint8_t)message.getBool());
    #ifdef MY_DEBUG
      Serial.print("--Incoming change for relay 3: ");
      Serial.println(message.getBool());
    #endif
  }
}
